@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2008  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2009  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,8 +25,6 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_pccmos.c,v 1.31.2.1 2008/01/18 19:12:29 debug Exp $
- *  
  *  COMMENT: PC CMOS/RTC device (ISA ports 0x70 and 0x71)
  *
  *  The main point of this device is to be a "PC style wrapper" for accessing
@@ -143,6 +141,7 @@ DEVINIT(pccmos)
 	case MACHINE_PREP:
 	case MACHINE_MVMEPPC:
 	case MACHINE_ALPHA:
+	case MACHINE_IYONIX:	// TODO: not sure about exact type.
 		break;
 	default:fatal("devinit_pccmos(): unimplemented machine type"
 		    " %i\n", devinit->machine->machine_type);
