@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2008  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2009  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,8 +25,6 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_pckbc.c,v 1.74.2.1 2008/01/18 19:12:29 debug Exp $
- *  
  *  COMMENT: 8042 PC keyboard controller (+ 8242WB Keyboard/Mouse controller)
  *
  *  This module includes emulation of the 8048 keyboard chip too.
@@ -872,14 +870,14 @@ if (x&1)
 		if (writeflag==MEM_READ) {
 			debug("[ pckbc: read from unimplemented reg %i ]\n",
 			    (int)relative_addr);
-			odata = d->reg[relative_addr];
+			odata = d->reg[relative_addr % DEV_PCKBC_LENGTH];
 		} else {
 			debug("[ pckbc: write to unimplemented reg %i:",
 			    (int)relative_addr);
 			for (i=0; i<len; i++)
 				debug(" %02x", data[i]);
 			debug(" ]\n");
-			d->reg[relative_addr] = idata;
+			d->reg[relative_addr % DEV_PCKBC_LENGTH] = idata;
 		}
 	}
 
